@@ -174,7 +174,6 @@ process_data_init (char *buf, char *cmd, char *word[],
 		switch (*cmd)
 		{
 		case 0:
-		 jump:
 			buf[j] = 0;
 			for (j = wordcount; j < PDIWORDS; j++)
 			{
@@ -207,12 +206,12 @@ process_data_init (char *buf, char *cmd, char *word[],
 					buf[j] = 0;
 					j++;
 
-					word[wordcount] = &buf[j];
-					word_eol[wordcount] = cmd + 1;
-					wordcount++;
-
-					if (wordcount == PDIWORDS - 1)
-						goto jump;
+					if (wordcount < PDIWORDS)
+					{
+						word[wordcount] = &buf[j];
+						word_eol[wordcount] = cmd + 1;
+						wordcount++;
+					}
 
 					space = TRUE;
 				}
@@ -330,41 +329,28 @@ cmd_away (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	GSList *list;
 	char *reason = word_eol[2];
-	int back = FALSE;
-	unsigned int gone;
 
-	if (!(*reason) && sess->server->is_away)
+	if (!(*reason))
 	{
-		/* mark back */
-		sess->server->p_set_back (sess->server);
-		back = TRUE;
-	} else
-	{
-		if (!(*reason))
+		if (sess->server->is_away)
 		{
-			if (sess->server->reconnect_away)
-				reason = sess->server->last_away_reason;
-			else
-				/* must manage memory pointed to by random_line() */
-				reason = random_line (prefs.awayreason);
+			if (sess->server->last_away_reason)
+				PrintTextf (sess, _("Already marked away: %s\n"), sess->server->last_away_reason);
+			return FALSE;
 		}
-		sess->server->p_set_away (sess->server, reason);
+
+		if (sess->server->reconnect_away)
+			reason = sess->server->last_away_reason;
+		else
+			/* must manage memory pointed to by random_line() */
+			reason = random_line (prefs.awayreason);
 	}
+	sess->server->p_set_away (sess->server, reason);
 
 	if (prefs.show_away_message)
 	{
-		if (back)
-		{
-			gone = time (NULL) - sess->server->away_time;
-			sprintf (tbuf, "me is back (gone %.2d:%.2d:%.2d)", gone / 3600,
-						(gone / 60) % 60, gone % 60);
-		} else
-		{
-			snprintf (tbuf, TBUFSIZE, "me is away: %s", reason);
-		}
-
-		list = sess_list;
-		while (list)
+		snprintf (tbuf, TBUFSIZE, "me is away: %s", reason);
+		for (list = sess_list; list; list = list->next)
 		{
 			/* am I the right server and not a dialog box */
 			if (((struct session *) list->data)->server == sess->server
@@ -373,19 +359,62 @@ cmd_away (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			{
 				handle_command ((session *) list->data, tbuf, TRUE);
 			}
-			list = list->next;
 		}
 	}
 
 	if (sess->server->last_away_reason != reason)
 	{
-		free (sess->server->last_away_reason);
+		if (sess->server->last_away_reason)
+			free (sess->server->last_away_reason);
+
 		if (reason == word_eol[2])
 			sess->server->last_away_reason = strdup (reason);
 		else
 			sess->server->last_away_reason = reason;
 	}
 
+	if (!sess->server->connected)
+		sess->server->reconnect_away = 1;
+
+	return TRUE;
+}
+
+static int
+cmd_back (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	GSList *list;
+	unsigned int gone;
+
+	if (sess->server->is_away)
+	{
+		sess->server->p_set_back (sess->server);
+
+		if (prefs.show_away_message)
+		{
+			gone = time (NULL) - sess->server->away_time;
+			sprintf (tbuf, "me is back (gone %.2d:%.2d:%.2d)", gone / 3600,
+						(gone / 60) % 60, gone % 60);
+			for (list = sess_list; list; list = list->next)
+			{
+				/* am I the right server and not a dialog box */
+				if (((struct session *) list->data)->server == sess->server
+					 && ((struct session *) list->data)->type == SESS_CHANNEL
+					 && ((struct session *) list->data)->channel[0])
+				{
+					handle_command ((session *) list->data, tbuf, TRUE);
+				}
+			}
+		}
+	}
+	else
+	{
+		PrintText (sess, _("Already marked back.\n"));
+	}
+
+	if (sess->server->last_away_reason)
+		free (sess->server->last_away_reason);
+	sess->server->last_away_reason = NULL;
+
 	return TRUE;
 }
 
@@ -547,40 +576,8 @@ cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_chanopt (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	int state;
-	if (!strcasecmp (word[3], "ON"))
-	{
-		state = TRUE;
-	} else if (!strcasecmp (word[3], "OFF"))
-	{
-		state = FALSE;
-	} else
-	{
-		if (!strcasecmp (word[2], "CONFMODE"))
-			state = sess->hide_join_part;
-		else if (!strcasecmp (word[2], "COLORPASTE"))
-			state = sess->color_paste;
-		else if (!strcasecmp (word[2], "BEEP"))
-			state = sess->beep;
-		else
-			return FALSE;
-
-		PrintTextf (sess, "%s is %s\n", word[2], state ? "ON" : "OFF");
-		return TRUE;
-	}
-
-	if (!strcasecmp (word[2], "CONFMODE"))
-	{
-		sess->hide_join_part = state;
-	} else if (!strcasecmp (word[2], "COLORPASTE"))
-	{
-		fe_set_color_paste (sess, state);
-	} else if (!strcasecmp (word[2], "BEEP"))
-	{
-		sess->beep = state;
-	}
-
-	return TRUE;
+	/* chanopt.c */
+	return chanopt_command (sess, tbuf, word, word_eol);
 }
 
 static int
@@ -620,12 +617,6 @@ cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	GSList *list = sess_list;
 	char *reason = word_eol[2];
 
-	if (reason[0] == 0)
-	{
-		fe_text_clear (sess);
-		return TRUE;
-	}
-
 	if (strcasecmp (reason, "HISTORY") == 0)
 	{
 		history_free (&sess->history);
@@ -638,13 +629,17 @@ cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			sess = list->data;
 			if (!sess->nick_said)
-				fe_text_clear (list->data);
+				fe_text_clear (list->data, 0);
 			list = list->next;
 		}
 		return TRUE;
 	}
 
-	return FALSE;
+	if (reason[0] != '-' && !isdigit (reason[0]) && reason[0] != 0)
+		return FALSE;
+
+	fe_text_clear (sess, atoi (reason));
+	return TRUE;
 }
 
 static int
@@ -736,7 +731,9 @@ static int
 cmd_cycle (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	char *key = sess->channelkey;
-	char *chan = sess->channel;
+	char *chan = word[2];
+	if (!*chan)
+		chan = sess->channel;
 	if (*chan && sess->type == SESS_CHANNEL)
 	{
 		sess->server->p_cycle (sess->server, chan, key);
@@ -1041,6 +1038,10 @@ menu_free (menu_entry *me)
 		free (me->cmd);
 	if (me->ucmd)
 		free (me->ucmd);
+	if (me->group)
+		free (me->group);
+	if (me->icon)
+		free (me->icon);
 	free (me);
 }
 
@@ -1143,8 +1144,27 @@ menu_del (char *path, char *label)
 	return 0;
 }
 
+static char
+menu_is_mainmenu_root (char *path, gint16 *offset)
+{
+	static const char *menus[] = {"\x4$TAB","\x5$TRAY","\x4$URL","\x5$NICK","\x5$CHAN"};
+	int i;
+
+	for (i = 0; i < 5; i++)
+	{
+		if (!strncmp (path, menus[i] + 1, menus[i][0]))
+		{
+			*offset = menus[i][0] + 1;	/* number of bytes to offset the root */
+			return 0;	/* is not main menu */
+		}
+	}
+
+	*offset = 0;
+	return 1;	/* is main menu */
+}
+
 static void
-menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int markup, int enable, int mod, int key)
+menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int markup, int enable, int mod, int key, char *group, char *icon)
 {
 	menu_entry *me;
 
@@ -1161,15 +1181,18 @@ menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, in
 
 	me = malloc (sizeof (menu_entry));
 	me->pos = pos;
+	me->modifier = mod;
+	me->is_main = menu_is_mainmenu_root (path, &me->root_offset);
 	me->state = state;
 	me->markup = markup;
 	me->enable = enable;
-	me->modifier = mod;
 	me->key = key;
 	me->path = strdup (path);
 	me->label = NULL;
 	me->cmd = NULL;
 	me->ucmd = NULL;
+	me->group = NULL;
+	me->icon = NULL;
 
 	if (label)
 		me->label = strdup (label);
@@ -1177,6 +1200,10 @@ menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, in
 		me->cmd = strdup (cmd);
 	if (ucmd)
 		me->ucmd = strdup (ucmd);
+	if (group)
+		me->group = strdup (group);
+	if (icon)
+		me->icon = strdup (icon);
 
 	menu_list = g_slist_append (menu_list, me);
 	label = fe_menu_add (me);
@@ -1194,7 +1221,7 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int idx = 2;
 	int len;
-	int pos = -1;
+	int pos = 0xffff;
 	int state;
 	int toggle = FALSE;
 	int enable = TRUE;
@@ -1202,6 +1229,8 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int key = 0;
 	int mod = 0;
 	char *label;
+	char *group = NULL;
+	char *icon = NULL;
 
 	if (!word[2][0] || !word[3][0])
 		return FALSE;
@@ -1213,6 +1242,13 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		idx++;
 	}
 
+	/* -i<ICONFILE> */
+	if (word[idx][0] == '-' && word[idx][1] == 'i')
+	{
+		icon = word[idx] + 2;
+		idx++;
+	}
+
 	/* -k<mod>,<key> key binding */
 	if (word[idx][0] == '-' && word[idx][1] == 'k')
 	{
@@ -1238,6 +1274,14 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		idx++;
 	}
 
+	/* -rSTATE,GROUP to specify a radio item */
+	if (word[idx][0] == '-' && word[idx][1] == 'r')
+	{
+		state = atoi (word[idx] + 2);
+		group = word[idx] + 4;
+		idx++;
+	}
+
 	/* -tX to specify toggle item with default state */
 	if (word[idx][0] == '-' && word[idx][1] == 't')
 	{
@@ -1246,6 +1290,9 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		toggle = TRUE;
 	}
 
+	if (word[idx+1][0] == 0)
+		return FALSE;
+
 	/* the path */
 	path_part (word[idx+1], tbuf, 512);
 	len = strlen (tbuf);
@@ -1269,13 +1316,13 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		if (toggle)
 		{
-			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, markup, enable, mod, key);
+			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, markup, enable, mod, key, NULL, NULL);
 		} else
 		{
 			if (word[idx + 2][0])
-				menu_add (tbuf, label, word[idx + 2], NULL, pos, 0, markup, enable, mod, key);
+				menu_add (tbuf, label, word[idx + 2], NULL, pos, state, markup, enable, mod, key, group, icon);
 			else
-				menu_add (tbuf, label, NULL, NULL, pos, 0, markup, enable, mod, key);
+				menu_add (tbuf, label, NULL, NULL, pos, state, markup, enable, mod, key, group, icon);
 		}
 		return TRUE;
 	}
@@ -1989,6 +2036,7 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	switch (str_ihash (word[2]))
 	{
+	case 0x058b836e: fe_ctrl_gui (sess, 8, 0); break; /* APPLY */
 	case 0xac1eee45: fe_ctrl_gui (sess, 7, 2); break; /* ATTACH */
 	case 0x05a72f63: fe_ctrl_gui (sess, 4, atoi (word[3])); break; /* COLOR */
 	case 0xb06a1793: fe_ctrl_gui (sess, 7, 1); break; /* DETACH */
@@ -2161,6 +2209,7 @@ cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int i;
 	int type = 0;
 	int quiet = 0;
+	char *mask;
 
 	if (!*word[2])
 	{
@@ -2177,17 +2226,26 @@ cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (type == 0)
 				return FALSE;
-			i = ignore_add (word[2], type);
+
+			mask = word[2];
+			if (strchr (mask, '?') == NULL &&
+			    strchr (mask, '*') == NULL &&
+			    userlist_find (sess, mask))
+			{
+				mask = tbuf;
+				snprintf (tbuf, TBUFSIZE, "%s!*@*", word[2]);
+			}
+
+			i = ignore_add (mask, type);
 			if (quiet)
 				return TRUE;
 			switch (i)
 			{
 			case 1:
-				EMIT_SIGNAL (XP_TE_IGNOREADD, sess, word[2], NULL, NULL, NULL, 0);
+				EMIT_SIGNAL (XP_TE_IGNOREADD, sess, mask, NULL, NULL, NULL, 0);
 				break;
 			case 2:	/* old ignore changed */
-				EMIT_SIGNAL (XP_TE_IGNORECHANGE, sess, word[2], NULL, NULL,
-								 NULL, 0);
+				EMIT_SIGNAL (XP_TE_IGNORECHANGE, sess, mask, NULL, NULL, NULL, 0);
 			}
 			return TRUE;
 		}
@@ -2321,7 +2379,7 @@ lastlog (session *sess, char *search, gboolean regexp)
 	lastlog_sess->lastlog_sess = sess;
 	lastlog_sess->lastlog_regexp = regexp;	/* remember the search type */
 
-	fe_text_clear (lastlog_sess);
+	fe_text_clear (lastlog_sess, 0);
 	fe_lastlog (sess, lastlog_sess, search, regexp);
 }
 
@@ -2343,16 +2401,43 @@ cmd_lastlog (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_list (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	sess->server->p_list_channels (sess->server, word_eol[2]);
+	sess->server->p_list_channels (sess->server, word_eol[2], 1);
 
 	return TRUE;
 }
 
+gboolean
+load_perform_file (session *sess, char *file)
+{
+	char tbuf[1024 + 4];
+	char *nl;
+	FILE *fp;
+
+	fp = xchat_fopen_file (file, "r", XOF_FULLPATH);
+	if (!fp)
+		return FALSE;
+
+	tbuf[1024] = 0;
+	while (fgets (tbuf, 1024, fp))
+	{
+		nl = strchr (tbuf, '\n');
+		if (nl == tbuf) /* skip empty commands */
+			continue;
+		if (nl)
+			*nl = 0;
+		if (tbuf[0] == prefs.cmdchar[0])
+			handle_command (sess, tbuf + 1, TRUE);
+		else
+			handle_command (sess, tbuf, TRUE);
+	}
+	fclose (fp);
+	return TRUE;
+}
+
 static int
 cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	FILE *fp;
-	char *error, *arg, *nl, *file;
+	char *error, *arg, *file;
 	int len;
 
 	if (!word[2][0])
@@ -2361,30 +2446,12 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (strcmp (word[2], "-e") == 0)
 	{
 		file = expand_homedir (word[3]);
-		fp = xchat_fopen_file (file, "r", XOF_FULLPATH);
-		if (!fp)
+		if (!load_perform_file (sess, file))
 		{
 			PrintTextf (sess, _("Cannot access %s\n"), file);
 			PrintText (sess, errorstring (errno));
-			free (file);
-			return TRUE;
 		}
 		free (file);
-
-		tbuf[1024] = 0;
-		while (fgets (tbuf, 1024, fp))
-		{
-			nl = strchr (tbuf, '\n');
-			if (nl == tbuf) /* skip empty commands */
-				continue;
-			if (nl)
-				*nl = 0;
-			if (tbuf[0] == prefs.cmdchar[0])
-				handle_command (sess, tbuf + 1, TRUE);
-			else
-				handle_command (sess, tbuf, TRUE);
-		}
-		fclose (fp);
 		return TRUE;
 	}
 
@@ -2647,8 +2714,14 @@ cmd_notify (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				EMIT_SIGNAL (XP_TE_DELNOTIFY, sess, word[i], NULL, NULL, NULL, 0);
 				return TRUE;
 			}
-			notify_adduser (word[i], net);
-			EMIT_SIGNAL (XP_TE_ADDNOTIFY, sess, word[i], NULL, NULL, NULL, 0);
+
+			if (net && strcmp (net, "ASK") == 0)
+				fe_notify_ask (word[i], NULL);
+			else
+			{
+				notify_adduser (word[i], net);
+				EMIT_SIGNAL (XP_TE_ADDNOTIFY, sess, word[i], NULL, NULL, NULL, 0);
+			}
 		}
 	} else
 		notify_showlist (sess);
@@ -2888,13 +2961,8 @@ cmd_settab (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_settext (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	if (*word_eol[2])
-	{
-		fe_set_inputbox_contents (sess, word_eol[2]);
-		return TRUE;
-	}
-
-	return FALSE;
+	fe_set_inputbox_contents (sess, word_eol[2]);
+	return TRUE;
 }
 
 static int
@@ -3072,6 +3140,47 @@ cmd_topic (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_tray (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (strcmp (word[2], "-b") == 0)
+	{
+		fe_tray_set_balloon (word[3], word[4][0] ? word[4] : NULL);
+		return TRUE;
+	}
+
+	if (strcmp (word[2], "-t") == 0)
+	{
+		fe_tray_set_tooltip (word[3][0] ? word[3] : NULL);
+		return TRUE;
+	}
+
+	if (strcmp (word[2], "-i") == 0)
+	{
+		fe_tray_set_icon (atoi (word[3]));
+		return TRUE;
+	}
+
+	if (strcmp (word[2], "-f") != 0)
+		return FALSE;
+
+	if (!word[3][0])
+	{
+		fe_tray_set_file (NULL);	/* default xchat icon */
+		return TRUE;
+	}
+
+	if (!word[4][0])
+	{
+		fe_tray_set_file (word[3]);	/* fixed custom icon */
+		return TRUE;
+	}
+
+	/* flash between 2 icons */
+	fe_tray_set_flash (word[4], word[5][0] ? word[5] : NULL, atoi (word[3]));
+	return TRUE;
+}
+
 static int
 cmd_unignore (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[])
@@ -3393,14 +3502,10 @@ const struct commands xc_cmds[] = {
 	{"ALLSERV", cmd_allservers, 0, 0, 1,
 	 N_("ALLSERV <cmd>, sends a command to all servers you're in")},
 	{"AWAY", cmd_away, 1, 0, 1, N_("AWAY [<reason>], sets you away")},
+	{"BACK", cmd_back, 1, 0, 1, N_("BACK, sets you back (not away)")},
 	{"BAN", cmd_ban, 1, 1, 1,
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
-	{"CHANOPT", cmd_chanopt, 0, 0, 1,
-	 N_("Set per channel options\n"
-	 "CHANOPT CONFMODE ON|OFF - Toggle conf mode/showing of join and part messages\n"
-	 "CHANOPT COLORPASTE ON|OFF - Toggle color paste\n"
-	 "CHANOPT BEEP ON|OFF - Toggle beep on message"
-	)},
+	{"CHANOPT", cmd_chanopt, 0, 0, 1, N_("CHANOPT [-quiet] <variable> [<value>]")},
 	{"CHARSET", cmd_charset, 0, 0, 1, 0},
 	{"CLEAR", cmd_clear, 0, 0, 1, N_("CLEAR [ALL|HISTORY], Clears the current text window or command history")},
 	{"CLOSE", cmd_close, 0, 0, 1, N_("CLOSE, Closes the current window/tab")},
@@ -3410,7 +3515,7 @@ const struct commands xc_cmds[] = {
 	{"CTCP", cmd_ctcp, 1, 0, 1,
 	 N_("CTCP <nick> <message>, send the CTCP message to nick, common messages are VERSION and USERINFO")},
 	{"CYCLE", cmd_cycle, 1, 1, 1,
-	 N_("CYCLE, parts current channel and immediately rejoins")},
+	 N_("CYCLE [<channel>], parts the current or given channel and immediately rejoins")},
 	{"DCC", cmd_dcc, 0, 0, 1,
 	 N_("\n"
 	 "DCC GET <nick>                      - accept an offered file\n"
@@ -3455,7 +3560,7 @@ const struct commands xc_cmds[] = {
 	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
 	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
 	{"GHOST", cmd_ghost, 1, 0, 1, N_("GHOST <nick> <password>, Kills a ghosted nickname")},
-	{"GUI", cmd_gui, 0, 0, 1, "GUI [ATTACH|DETACH|SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]\n"
+	{"GUI", cmd_gui, 0, 0, 1, "GUI [APPLY|ATTACH|DETACH|SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]\n"
 									  "       GUI [MSGBOX <text>|MENU TOGGLE]"},
 	{"HELP", cmd_help, 0, 0, 1, 0},
 	{"HOP", cmd_hop, 1, 1, 1,
@@ -3489,7 +3594,8 @@ const struct commands xc_cmds[] = {
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
 	{"ME", cmd_me, 0, 0, 1,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
-	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-k<mod>,<key>] [-m] [-pX] [-tX] {ADD|DEL} <path> [command] [unselect command]"},
+	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-i<ICONFILE>] [-k<mod>,<key>] [-m] [-pX] [-r<X,group>] [-tX] {ADD|DEL} <path> [command] [unselect command]\n"
+										 "       See http://xchat.org/docs/menu/ for more details."},
 	{"MKICK", cmd_mkick, 1, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
 	{"MODE", cmd_mode, 1, 0, 1, 0},
@@ -3510,7 +3616,7 @@ const struct commands xc_cmds[] = {
 	 N_("NOTIFY [-n network1[,network2,...]] [<nick>], displays your notify list or adds someone to it")},
 	{"OP", cmd_op, 1, 1, 1,
 	 N_("OP <nick>, gives chanop status to the nick (needs chanop)")},
-	{"PART", cmd_part, 1, 1, 1,
+	{"PART", cmd_part, 1, 1, 0,
 	 N_("PART [<channel>] [<reason>], leaves the channel, by default the current one")},
 	{"PING", cmd_ping, 1, 0, 1,
 	 N_("PING <nick | channel>, CTCP pings nick or channel")},
@@ -3546,13 +3652,20 @@ const struct commands xc_cmds[] = {
 	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
-	{"SET", cmd_set, 0, 0, 1, N_("SET [-e] [-quiet] <variable> [<value>]")},
+	{"SET", cmd_set, 0, 0, 1, N_("SET [-e] [-or] [-quiet] <variable> [<value>]")},
 	{"SETCURSOR", cmd_setcursor, 0, 0, 1, N_("SETCURSOR [-|+]<position>")},
 	{"SETTAB", cmd_settab, 0, 0, 1, 0},
 	{"SETTEXT", cmd_settext, 0, 0, 1, 0},
 	{"SPLAY", cmd_splay, 0, 0, 1, "SPLAY <soundfile>"},
 	{"TOPIC", cmd_topic, 1, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
+	{"TRAY", cmd_tray, 0, 0, 1,
+	 N_("\nTRAY -f <timeout> <file1> [<file2>] Blink tray between two icons.\n"
+		   "TRAY -f <filename>                  Set tray to a fixed icon.\n"
+			"TRAY -i <number>                    Blink tray with an internal icon.\n"
+			"TRAY -t <text>                      Set the tray tooltip.\n"
+			"TRAY -b <title> <text>              Set the tray balloon."
+			)},
 	{"UNBAN", cmd_unban, 1, 1, 1,
 	 N_("UNBAN <mask> [<mask>...], unbans the specified masks.")},
 	{"UNIGNORE", cmd_unignore, 0, 0, 1, N_("UNIGNORE <mask> [QUIET]")},
@@ -3697,6 +3810,7 @@ auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
 						return 2;
 					dest[0] = '%';
 					dest[1] = 0;
+					dest++;
 					break;
 				case 'a':
 					utf = a; break;
@@ -3953,8 +4067,8 @@ static void
 handle_say (session *sess, char *text, int check_spch)
 {
 	struct DCC *dcc;
-	char *word[PDIWORDS];
-	char *word_eol[PDIWORDS];
+	char *word[PDIWORDS+1];
+	char *word_eol[PDIWORDS+1];
 	char pdibuf_static[1024];
 	char newcmd_static[1024];
 	char *pdibuf = pdibuf_static;
@@ -3978,6 +4092,10 @@ handle_say (session *sess, char *text, int check_spch)
 	if (check_spch && prefs.perc_color)
 		check_special_chars (text, prefs.perc_ascii);
 
+	/* Python relies on this */
+	word[PDIWORDS] = NULL;
+	word_eol[PDIWORDS] = NULL;
+
 	/* split the text into words and word_eol */
 	process_data_init (pdibuf, text, word, word_eol, TRUE, FALSE);
 
@@ -4010,7 +4128,7 @@ handle_say (session *sess, char *text, int check_spch)
 		{
 			inbound_chanmsg (sess->server, NULL, sess->channel,
 								  sess->server->nick, text, TRUE, FALSE);
-			set_topic (sess, net_ip (dcc->addr));
+			set_topic (sess, net_ip (dcc->addr), net_ip (dcc->addr));
 			goto xit;
 		}
 	}
@@ -4083,8 +4201,8 @@ handle_command (session *sess, char *cmd, int check_spch)
 	struct popup *pop;
 	int user_cmd = FALSE;
 	GSList *list;
-	char *word[PDIWORDS];
-	char *word_eol[PDIWORDS];
+	char *word[PDIWORDS+1];
+	char *word_eol[PDIWORDS+1];
 	static int command_level = 0;
 	struct commands *int_cmd;
 	char pdibuf_static[1024];
@@ -4115,6 +4233,12 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 	/* split the text into words and word_eol */
 	process_data_init (pdibuf, cmd, word, word_eol, TRUE, TRUE);
+
+	/* ensure an empty string at index 32 for cmd_deop etc */
+	/* (internal use only, plugins can still only read 1-31). */
+	word[PDIWORDS] = "\000\000";
+	word_eol[PDIWORDS] = "\000\000";
+
 	int_cmd = find_internal_command (word[1]);
 	/* redo it without quotes processing, for some commands like /JOIN */
 	if (int_cmd && !int_cmd->handle_quotes)
