@@ -153,8 +153,14 @@ struct xchatprefs
 
 	int away_timeout;
 	int away_size_max;
-	int paned_pos;
-	int tabs_position;
+
+	int gui_pane_left_size;
+	int gui_pane_right_size;
+
+	int gui_ulist_pos;
+	int tab_pos;
+
+	int _tabs_position;
 	int tab_layout;
 	int max_auto_indent;
 	int dcc_blocksize;
@@ -175,6 +181,7 @@ struct xchatprefs
 	int gui_url_mod;
 	int gui_usermenu;
 	int gui_join_dialog;
+	int gui_quit_dialog;
 	int dialog_left;
 	int dialog_top;
 	int dialog_width;
@@ -189,7 +196,6 @@ struct xchatprefs
 
 	unsigned int tab_small;
 	unsigned int tab_sort;
-	unsigned int tab_icons;
 	unsigned int mainwindow_save;
 	unsigned int perc_color;
 	unsigned int perc_ascii;
@@ -226,11 +232,23 @@ struct xchatprefs
 	unsigned int slist_skip;
 	unsigned int slist_select;
 	unsigned int filterbeep;
-	unsigned int beepmsg;
-	unsigned int beepchans;
-	unsigned int beephilight;
+
+	unsigned int input_balloon_chans;
+	unsigned int input_balloon_hilight;
+	unsigned int input_balloon_priv;
+
+	unsigned int input_beep_chans;
+	unsigned int input_beep_hilight;
+	unsigned int input_beep_priv;
+
+	unsigned int input_flash_chans;
 	unsigned int input_flash_hilight;
 	unsigned int input_flash_priv;
+
+	unsigned int input_tray_chans;
+	unsigned int input_tray_hilight;
+	unsigned int input_tray_priv;
+
 	unsigned int truncchans;
 	unsigned int privmsgtab;
 	unsigned int irc_join_delay;
@@ -253,6 +271,7 @@ struct xchatprefs
 	unsigned int style_inputbox;
 	unsigned int windows_as_tabs;
 	unsigned int indent_nicks;
+	unsigned int text_replay;
 	unsigned int show_marker;
 	unsigned int show_separator;
 	unsigned int thin_separator;
@@ -260,8 +279,9 @@ struct xchatprefs
 	unsigned int wordwrap;
 	unsigned int gui_input_spell;
 	unsigned int gui_tray;
-	unsigned int gui_tray_blink;
 	unsigned int gui_tray_flags;
+	unsigned int gui_tweaks;
+	unsigned int _gui_ulist_left;
 	unsigned int throttle;
 	unsigned int topicbar;
 	unsigned int hideuserlist;
@@ -295,8 +315,24 @@ struct xchatprefs
 #define SESS_NOTICES	4
 #define SESS_SNOTICES	5
 
+/* Per-Channel Settings */
+#define SET_OFF 0
+#define SET_ON 1
+#define SET_DEFAULT 2 /* use global setting */
+
 typedef struct session
 {
+	/* Per-Channel Alerts */
+	/* use a byte, because we need a pointer to each element */
+	guint8 alert_beep;
+	guint8 alert_taskbar;
+	guint8 alert_tray;
+
+	/* Per-Channel Settings */
+	guint8 text_hidejoinpart;
+	guint8 text_logging;
+	guint8 text_scrollback;
+
 	struct server *server;
 	void *usertree_alpha;			/* pure alphabetical tree */
 	void *usertree;					/* ordered with Ops first */
@@ -307,6 +343,8 @@ typedef struct session
 	char channelkey[64];			  /* XXX correct max length? */
 	int limit;						  /* channel user limit */
 	int logfd;
+	int scrollfd;							/* scrollback filedes */
+	int scrollwritten;					/* number of lines written */
 
 	char lastnick[NICKLEN];			  /* last nick you /msg'ed */
 
@@ -329,21 +367,17 @@ typedef struct session
 	struct session_gui *gui;		/* initialized by fe_new_window */
 	struct restore_gui *res;
 
-	int userlisthidden;
-	int type;
+	int type;					/* SESS_* */
+
 	int new_data:1;			/* new data avail? (purple tab) */
 	int nick_said:1;		/* your nick mentioned? (blue tab) */
 	int msg_said:1;			/* new msg available? (red tab) */
+
 	int ignore_date:1;
 	int ignore_mode:1;
 	int ignore_names:1;
 	int end_of_names:1;
 	int doing_who:1;		/* /who sent on this channel */
-	/* these are in the bottom-right menu */
-	unsigned int hide_join_part:1;	/* hide join & part messages? */
-	unsigned int beep:1;				/* beep enabled? */
-	unsigned int tray:1;				/* tray flash for this chan? */
-	unsigned int color_paste:1;
 	int done_away_check:1;	/* done checking for away status changes */
 	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
 } session;
@@ -376,6 +410,7 @@ typedef struct server
 	void (*p_ns_identify)(struct server *, char *pass);
 	void (*p_ns_ghost)(struct server *, char *usname, char *pass);
 	void (*p_join)(struct server *, char *channel, char *key);
+	void (*p_join_list)(struct server *, GSList *channels, GSList *keys);
 	void (*p_login)(struct server *, char *user, char *realname);
 	void (*p_join_info)(struct server *, char *channel);
 	void (*p_mode)(struct server *, char *target, char *mode);
@@ -410,6 +445,8 @@ typedef struct server
 #ifdef USE_OPENSSL
 	SSL *ssl;
 	int ssl_do_connect_tag;
+#else
+	void *ssl;
 #endif
 	int childread;
 	int childwrite;
@@ -459,32 +496,36 @@ typedef struct server
 	time_t away_time;					/* when we were marked away */
 
 	char *encoding;					/* NULL for system */
+	char *autojoin;			/* list of channels & keys to join */
 
-	int motd_skipped:1;
+	unsigned int motd_skipped:1;
 	unsigned int connected:1;
 	unsigned int connecting:1;
-	int no_login:1;
-	int skip_next_userhost:1;			/* used for "get my ip from server" */
-	int inside_whois:1;
-	int doing_dns:1;				/* /dns has been done */
-	unsigned int end_of_motd:1;	/* end of motd reached (logged in) */
-	int sent_quit:1;				/* sent a QUIT already? */
-	int use_listargs:1;			/* undernet and dalnet need /list >0,<10000 */
+	unsigned int no_login:1;
+	unsigned int skip_next_userhost:1;/* used for "get my ip from server" */
+	unsigned int skip_next_whois:1;	/* hide whois output */
+	unsigned int inside_whois:1;
+	unsigned int doing_dns:1;			/* /dns has been done */
+	unsigned int end_of_motd:1;		/* end of motd reached (logged in) */
+	unsigned int sent_quit:1;			/* sent a QUIT already? */
+	unsigned int use_listargs:1;		/* undernet and dalnet need /list >0,<10000 */
 	unsigned int is_away:1;
-	int reconnect_away:1;		/* whether to reconnect in is_away state */
-	int dont_use_proxy:1;		/* to proxy or not to proxy */
-	int supports_watch:1;		/* supports the WATCH command */
-	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
-	unsigned int have_whox:1;	/* have undernet's WHOX features */
-	unsigned int have_capab:1;	/* supports CAPAB (005 tells us) */
-	unsigned int have_idmsg:1;	/* freenode's IDENTIFY-MSG */
+	unsigned int reconnect_away:1;	/* whether to reconnect in is_away state */
+	unsigned int dont_use_proxy:1;	/* to proxy or not to proxy */
+	unsigned int supports_watch:1;	/* supports the WATCH command */
+	unsigned int bad_prefix:1;			/* gave us a bad PREFIX= 005 number */
+	unsigned int have_namesx:1;		/* 005 tokens NAMESX and UHNAMES */
+	unsigned int have_uhnames:1;
+	unsigned int have_whox:1;		/* have undernet's WHOX features */
+	unsigned int have_capab:1;		/* supports CAPAB (005 tells us) */
+	unsigned int have_idmsg:1;		/* freenode's IDENTIFY-MSG */
 	unsigned int have_except:1;	/* ban exemptions +e */
 	unsigned int using_cp1255:1;	/* encoding is CP1255/WINDOWS-1255? */
 	unsigned int using_irc:1;		/* encoding is "IRC" (CP1252/UTF-8 hybrid)? */
-	int use_who:1;				/* whether to use WHO command to get dcc_ip */
+	unsigned int use_who:1;			/* whether to use WHO command to get dcc_ip */
 #ifdef USE_OPENSSL
-	int use_ssl:1;					  /* is server SSL capable? */
-	int accept_invalid_cert:1;	  /* ignore result of server's cert. verify */
+	unsigned int use_ssl:1;				  /* is server SSL capable? */
+	unsigned int accept_invalid_cert:1;/* ignore result of server's cert. verify */
 #endif
 } server;
 
