@@ -32,10 +32,12 @@
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkliststore.h>
+#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktreeselection.h>
 
 #include "../common/xchat.h"
+#include "../common/fe.h"
 #include "../common/modes.h"
 #include "../common/outbound.h"
 #include "../common/xchatc.h"
@@ -64,20 +66,51 @@ get_store (struct session *sess)
 	return GTK_LIST_STORE (gtk_tree_view_get_model (get_view (sess)));
 }
 
-void
-fe_add_ban_list (struct session *sess, char *mask, char *who, char *when)
+static gboolean
+supports_exempt (server *serv)
 {
-	GtkListStore *store;
-	GtkTreeIter iter;
+	char *cm = serv->chanmodes;
 
-	store = get_store (sess);
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, mask, 1, who, 2, when, -1);
+	if (serv->have_except)
+		return TRUE;
 
+	if (!cm)
+		return FALSE;
+
+	while (*cm)
+	{
+		if (*cm == ',')
+			break;
+		if (*cm == 'e')
+			return TRUE;
+		cm++;
+	}
+
+	return FALSE;
 }
 
 void
-fe_ban_list_end (struct session *sess)
+fe_add_ban_list (struct session *sess, char *mask, char *who, char *when, int is_exempt)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	char buf[512];
+
+	store = get_store (sess);
+	gtk_list_store_append (store, &iter);
+
+	if (is_exempt)
+	{
+		snprintf (buf, sizeof (buf), "(EX) %s", mask);
+		gtk_list_store_set (store, &iter, 0, buf, 1, who, 2, when, -1);
+	} else
+	{
+		gtk_list_store_set (store, &iter, 0, mask, 1, who, 2, when, -1);
+	}
+}
+
+void
+fe_ban_list_end (struct session *sess, int is_exemption)
 {
 	gtk_widget_set_sensitive (sess->res->banlist_butRefresh, TRUE);
 }
@@ -95,7 +128,7 @@ banlist_do_refresh (struct session *sess)
 
 		gtk_widget_set_sensitive (sess->res->banlist_butRefresh, FALSE);
 
-		snprintf (tbuf, sizeof tbuf, "X-Chat: Ban List (%s, %s)",
+		snprintf (tbuf, sizeof tbuf, "XChat: Ban List (%s, %s)",
 						sess->channel, sess->server->servername);
 		mg_set_title (sess->res->banlist_window, tbuf);
 
@@ -103,8 +136,20 @@ banlist_do_refresh (struct session *sess)
 		gtk_list_store_clear (store);
 
 		handle_command (sess, "ban", FALSE);
+#ifdef WIN32
+		if (0)
+#else
+		if (supports_exempt (sess->server))
+		{
+#endif
+			snprintf (tbuf, sizeof (tbuf), "quote mode %s +e", sess->channel);
+			handle_command (sess, tbuf, FALSE);
+		}
+
 	} else
-		gtkutil_simpledialog ("Not connected.");
+	{
+		fe_message (_("Not connected."), FE_MSG_ERROR);
+	}
 }
 
 static void
@@ -119,14 +164,14 @@ banlist_refresh (GtkWidget * wid, struct session *sess)
 	banlist_do_refresh (sess);
 }
 
-static void
-banlist_unban (GtkWidget * wid, struct session *sess)
+static int
+banlist_unban_inner (gpointer none, struct session *sess, int do_exempts)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *sel;
 	GtkTreeIter iter;
 	char tbuf[2048];
-	char **masks;
+	char **masks, *tmp, *space;
 	int num_sel, i;
 
 	/* grab the list of selected items */
@@ -144,13 +189,10 @@ banlist_unban (GtkWidget * wid, struct session *sess)
 	}
 
 	if (num_sel < 1)
-	{
-		gtkutil_simpledialog (_("You must select some bans."));
-		return;
-	}
+		return 0;
 
 	/* create an array of all the masks */
-	masks = malloc (num_sel * sizeof (char *));
+	masks = calloc (1, num_sel * sizeof (char *));
 
 	i = 0;
 	gtk_tree_model_get_iter_first (model, &iter);
@@ -159,30 +201,85 @@ banlist_unban (GtkWidget * wid, struct session *sess)
 		if (gtk_tree_selection_iter_is_selected (sel, &iter))
 		{
 			gtk_tree_model_get (model, &iter, MASK_COLUMN, &masks[i], -1);
-			i++;
+			space = strchr (masks[i], ' ');
+
+			if (do_exempts)
+			{
+				if (space)
+				{
+					/* remove the "(EX) " */
+					tmp = masks[i];
+					masks[i] = g_strdup (space + 1);
+					g_free (tmp);
+					i++;
+				}
+			} else
+			{
+				if (!space)
+					i++;
+			}
 		}
 	}
 	while (gtk_tree_model_iter_next (model, &iter));
 
 	/* and send to server */
-	send_channel_modes (sess, tbuf, masks, 0, i, '-', 'b', 0);
+	if (do_exempts)
+		send_channel_modes (sess, tbuf, masks, 0, i, '-', 'e', 0);
+	else
+		send_channel_modes (sess, tbuf, masks, 0, i, '-', 'b', 0);
 
 	/* now free everything, and refresh banlist */	
 	for (i=0; i < num_sel; i++)
 		g_free (masks[i]);
 	free (masks);
 
+	return num_sel;
+}
+
+static void
+banlist_unban (GtkWidget * wid, struct session *sess)
+{
+	int num = 0;
+
+	num += banlist_unban_inner (wid, sess, FALSE);
+	num += banlist_unban_inner (wid, sess, TRUE);
+
+	if (num < 1)
+	{
+		fe_message (_("You must select some bans."), FE_MSG_ERROR);
+		return;
+	}
+
 	banlist_do_refresh (sess);
 }
 
+static void
+banlist_clear_cb (GtkDialog *dialog, gint response, gpointer sess)
+{
+	GtkTreeSelection *sel;
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	if (response == GTK_RESPONSE_OK)
+	{
+		sel = gtk_tree_view_get_selection (get_view (sess));
+		gtk_tree_selection_select_all (sel);
+		banlist_unban (NULL, sess);
+	}
+}
+
 static void
 banlist_clear (GtkWidget * wid, struct session *sess)
 {
-	GtkTreeSelection *sel;
+	GtkWidget *dialog;
 
-	sel = gtk_tree_view_get_selection (get_view (sess));
-	gtk_tree_selection_select_all (sel);
-	banlist_unban (wid, sess);
+	dialog = gtk_message_dialog_new (NULL, 0,
+								GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL,
+					_("Are you sure you want to remove all bans in %s?"), sess->channel);
+	g_signal_connect (G_OBJECT (dialog), "response",
+							G_CALLBACK (banlist_clear_cb), sess);
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+	gtk_widget_show (dialog);
 }
 
 static void
@@ -227,10 +324,9 @@ banlist_crop (GtkWidget * wid, struct session *sess)
 		g_slist_foreach (list, (GFunc)g_free, NULL);
 		g_slist_free (list);
 
-		banlist_unban (wid, sess);
+		banlist_unban (NULL, sess);
 	} else
-		gtkutil_simpledialog (_("You must select some bans."));
-
+		fe_message (_("You must select some bans."), FE_MSG_ERROR);
 }
 
 static GtkWidget *
@@ -289,12 +385,15 @@ banlist_opengui (struct session *sess)
 	}
 
 	if (sess->type != SESS_CHANNEL)
+	{
+		fe_message (_("You can only open the Ban List window while in a channel tab."), FE_MSG_ERROR);
 		return;
+	}
 
-	snprintf (tbuf, sizeof tbuf, _("X-Chat: Ban List (%s)"),
+	snprintf (tbuf, sizeof tbuf, _("XChat: Ban List (%s)"),
 					sess->server->servername);
 
-	sess->res->banlist_window = mg_create_generic_tab ("banlist", tbuf, FALSE,
+	sess->res->banlist_window = mg_create_generic_tab ("BanList", tbuf, FALSE,
 					TRUE, banlist_closegui, sess, 550, 200, &vbox1, sess->server);
 
 	/* create banlist view */
@@ -307,7 +406,7 @@ banlist_opengui (struct session *sess)
 	gtk_widget_show (bbox);
 
 	gtkutil_button (bbox, GTK_STOCK_REMOVE, 0, banlist_unban, sess,
-	                _("Unban"));
+	                _("Remove"));
 	gtkutil_button (bbox, GTK_STOCK_REMOVE, 0, banlist_crop, sess,
 	                _("Crop"));
 	gtkutil_button (bbox, GTK_STOCK_CLEAR, 0, banlist_clear, sess,
