@@ -41,6 +41,10 @@ GdkPixbuf *pix_op;
 GdkPixbuf *pix_hop;
 GdkPixbuf *pix_voice;
 
+GdkPixbuf *pix_tray_msg;
+GdkPixbuf *pix_tray_hilight;
+GdkPixbuf *pix_tray_file;
+
 GdkPixbuf *pix_channel;
 GdkPixbuf *pix_dialog;
 GdkPixbuf *pix_server;
@@ -84,12 +88,12 @@ pixmap_load_from_file (char *filename)
 }
 
 #define LOADPIX(vv,pp,ff) \
-	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/"ff, 0); \
+	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/xchat/"ff, 0); \
 	if (!vv) \
 		vv = gdk_pixbuf_new_from_inline (-1, pp, FALSE, 0);
 
 #define LOADPIX_DISKONLY(vv,ff) \
-	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/"ff, 0);
+	vv = gdk_pixbuf_new_from_file (XCHATSHAREDIR"/xchat/"ff, 0);
 
 #define EXT ".png"
 
@@ -97,7 +101,9 @@ void
 pixmaps_init (void)
 {
 	pix_book = gdk_pixbuf_new_from_inline (-1, bookpng, FALSE, 0);
-	pix_xchat = gdk_pixbuf_new_from_inline (-1, xchatpng, FALSE, 0);
+
+	/* used in About window, tray icon and WindowManager icon. */
+	LOADPIX (pix_xchat, xchatpng, "xchat"EXT);
 
 	/* userlist icons, with inlined defaults */
 	LOADPIX (pix_hop, hoppng, "hop"EXT);
@@ -106,6 +112,11 @@ pixmaps_init (void)
 	LOADPIX (pix_op, oppng, "op"EXT);
 	LOADPIX (pix_voice, voicepng, "voice"EXT);
 
+	/* tray icons, with inlined defaults */
+	LOADPIX (pix_tray_msg, traymsgpng, "message"EXT);
+	LOADPIX (pix_tray_hilight, trayhilightpng, "highlight"EXT);
+	LOADPIX (pix_tray_file, trayfilepng, "fileoffer"EXT);
+
 	/* treeview icons, no defaults, load from disk only */
 	LOADPIX_DISKONLY (pix_channel,	"channel"EXT);
 	LOADPIX_DISKONLY (pix_dialog,		"dialog"EXT);
