@@ -106,7 +106,7 @@ fe_new_window (struct session *sess, int focus)
 				" \017This binary compiled \00310"__DATE__"\017\n",
 				get_cpu_str(),
 				glib_major_version, glib_minor_version, glib_micro_version);
-	fe_print_text (sess, buf);
+	fe_print_text (sess, buf, 0);
 
 	fe_print_text (sess, "\n\nCompiled in Features\0032:\017 "
 #ifdef USE_PLUGIN
@@ -121,7 +121,7 @@ fe_new_window (struct session *sess, int focus)
 #ifdef USE_IPV6
 	"IPv6"
 #endif
-	"\n\n");
+	"\n\n", 0);
 	fflush (stdout);
 	fflush (stdin);
 }
@@ -146,7 +146,7 @@ timecat (char *buf)
 static const short colconv[] = { 0, 7, 4, 2, 1, 3, 5, 11, 13, 12, 6, 16, 14, 15, 10, 7 };
 
 void
-fe_print_text (struct session *sess, char *text)
+fe_print_text (struct session *sess, char *text, time_t stamp)
 {
 	int dotime = FALSE;
 	char num[8];
@@ -572,7 +572,7 @@ fe_add_rawlog (struct server *serv, char *text, int len, int outbound)
 {
 }
 void
-fe_set_topic (struct session *sess, char *topic)
+fe_set_topic (struct session *sess, char *topic, char *stripped_topic)
 {
 }
 void
@@ -584,7 +584,7 @@ fe_set_hilight (struct session *sess)
 {
 }
 void
-fe_set_tab_color (struct session *sess, int col, int flash)
+fe_set_tab_color (struct session *sess, int col)
 {
 }
 void
@@ -631,7 +631,11 @@ fe_notify_update (char *name)
 {
 }
 void
-fe_text_clear (struct session *sess)
+fe_notify_ask (char *name, char *networks)
+{
+}
+void
+fe_text_clear (struct session *sess, int lines)
 {
 }
 void
@@ -759,7 +763,7 @@ fe_userlist_hide (session * sess)
 {
 }
 void
-fe_lastlog (session * sess, session * lastlog_sess, char *sstr)
+fe_lastlog (session * sess, session * lastlog_sess, char *sstr, gboolean regexp)
 {
 }
 void
@@ -775,10 +779,6 @@ fe_set_away (server *serv)
 {
 }
 void
-fe_set_color_paste (session *sess, int status)
-{
-}
-void
 fe_serverlist_open (session *sess)
 {
 }
@@ -795,7 +795,7 @@ fe_idle_add (void *func, void *data)
 {
 }
 void
-fe_ctrl_gui (session *sess, int action, int arg)
+fe_ctrl_gui (session *sess, fe_gui_action action, int arg)
 {
 }
 int
@@ -845,3 +845,17 @@ void
 fe_server_event (server *serv, int type, int arg)
 {
 }
+void
+fe_flash_window (struct session *sess)
+{
+}
+void fe_get_file (const char *title, char *initial,
+				 void (*callback) (void *userdata, char *file), void *userdata,
+				 int flags)
+{
+}
+void fe_tray_set_flash (const char *filename1, const char *filename2, int timeout){}
+void fe_tray_set_file (const char *filename){}
+void fe_tray_set_icon (feicon icon){}
+void fe_tray_set_tooltip (const char *text){}
+void fe_tray_set_balloon (const char *title, const char *text){}
